# sb::Data::Image
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Data::Image;
use strict;
use sb::Config ();
use sb::File::Object ();
use sb::Data::Object ();
use base qw( sb::Data::Object );
use vars qw( $VERSION );
$VERSION = '0.16';
# 0.16 [2009/01/24] changed THUMB_PREFIX
# 0.15 [2009/01/24] modified set_file to add new option to set thumbnail name without id
# 0.14 [2008/11/14] modified get_as_tag to support extra attributes
# 0.13 [2008/10/23] modified set_file to extract uploaded directory correctly
# 0.12 [2008/10/22] modified get_as_tag to get a correct path of the small image
# 0.11 [2007/09/24] added set_file
# 0.10 [2007/08/04] added initializer
# 0.09 [2007/07/16] changed return to make them consistent
# 0.08 [2007/07/09] got rid of "return( undef )" completely
# 0.07 [2007/02/11] ported from biscuit
# 0.06 [2006/07/04] got rid of "return( undef )"
# 0.05 [2006/02/01] changed _file_contenttype
# 0.04 [2005/08/02] added some functions to calculate image size
# 0.03 [2005/07/26] changed _file_extension/_file_contenttype to add some file types.
# 0.02 [2005/07/22] changed data structure to array
# 0.01 [2005/07/16] changed upload to create thumbnail correctly
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines / static variables
sub IMAGE_PREFIX     (){ 'file' };
sub THUMB_PREFIX     (){ 'thm' };
sub DEFAULT_FILENAME (){ 'file' };
sub PARTS_DIR        (){ 'icon/' };
sub ICON_UNDEFINED   (){ 'undefined.gif' };
sub ICON_SUFFIX      (){ '.gif' };
sub ICON_WIDTH       (){ 32 };
sub ICON_HEIGHT      (){ 32 };
sub THUMB_CLASS      (){ 'thumb' };
sub IMAGE_CLASS      (){ 'pict' };
# --------------------------------------
# // override base class - table
sub table
{
	my $self = shift;
	return (
		'id'     => 'uni_id',   # id
		'wid'    => 'id',       # weblog id (reserved)
		'auth'   => 'id',       # creator(author)
		'date'   => 'date',     # date of creation
		'name'   => 'str(255)', # name
		'file'   => 'str(255)', # file name
		'thumb'  => 'str(255)', # thumbnail image name if it has a thumbnail image
		'stat'   => 'int',      # flag for image selector
		'icon_c' => 'int',      # flag for comment icon
		'icon_t' => 'int',      # flag for trackback icon
		'dir'    => 'str(255)', # uploaded directory
		'eid'    => 'str(255)', # artilcles which are included this file
		'tz'     => 'zone',     # timezone
	);
}
# --------------------------------------
# // initializer
BEGIN
{
	__PACKAGE__->init_accessor();
}
sub default
{ # default settings
	my $self  = shift;
	my %param = @_;
	$param{'wid'} |= 0;
	$param{'auth'} |= 0;
	$param{'stat'} |= 0;
	$param{'icon_c'} |= 0;
	$param{'icon_t'} |= 0;
	$self->SUPER::initialize(%param);
}
# --------------------------------------
# // override base class
sub erase
{
	my $self = shift;
	$self->delete_file();
	$self->SUPER::erase;
}
# --------------------------------------
# // public functions
sub dir
{
	my $self = shift;
	$self->{'dir'} = shift if @_;
	return ($self->{'dir'} ne '') 
		? $self->{'dir'} 
		: sb::Config->get()->value('conf_dir_img');
}
sub object
{
	my $self = shift;
	my $type = $self->_check_type(shift);
	return sb::File::Object->new('fullpath'=>$self->get_path($type));
}
sub rename_file
{
	my $self = shift;
	my %param = (
		'dir'   => $self->dir(),
		'file'  => $self->file(),
		'thumb' => $self->thumb(),
		@_
	);
	if (  $param{'dir'} eq $self->dir() 
	  and $param{'file'} eq $self->file() 
	  and $param{'thumb'} eq $self->thumb() )
	{
		return;
	}
	my $old_dir = sb::Config->get()->value('conf_dir_base') . $self->dir();
	foreach my $type ('file','thumb')
	{
		next if ($self->$type() eq '');
		my $old = $old_dir . $self->$type();
		my $new = sb::Config->get()->value('conf_dir_base') . $param{'dir'} . $param{$type};
		if ( (-e $old and rename($old,$new)) or (!-e $old and -e $new) )
		{
			$self->$type($param{$type});
			$self->dir($param{'dir'});
		}
	}
	return 1;
}
sub delete_file
{
	my $self = shift;
	unlink( $self->get_path('file') );
	unlink( $self->get_path('thumb') ) if ($self->thumb() ne '');
}
sub set_file
{
	my $self = shift;
	my $file = shift;
	my $create_thumb = shift;
	my $without_id = shift;
	my $base = sb::Config->get()->value('conf_dir_base');
	my $dir  = $file->path();
	$dir =~ s/^\Q$base\E//;
	$self->dir($dir);
	$self->file($file->name());
	return if (!$create_thumb);
	my $thumb = THUMB_PREFIX;
	$thumb .= $self->id() if ( !$without_id );
	$thumb .= '_' . $file->name_without_ext();
	if ( $file->is_image() )
	{ # --- image file
		my $size = sb::Config->get()->value('conf_thumbsize');
		my $small = $file->create_smallimage(
			'path' => $self->dir(),
			'name' => $thumb . $file->extension(),
			'size' => [$file->get_size('max_w'=>$size,'max_h'=>$size)],
		);
		$self->thumb($thumb . $file->extension()) if ($small);
	}
	elsif ( $file->is_audio() )
	{ # --- audio file
		my $data = $file->audio_info('artwork');
		if ($data ne '')
		{
			my $artwork = Spica::File::Object->new(
				'path' => $self->dir(),
				'name' => $thumb . '.' . $file->audio_info('arttype'),
			);
			$self->thumb($artwork->name()) if ($artwork->set_data($data));
		}
	}
}
sub get_as_tag
{
	my $self = shift;
	my %param = (
		'max_w' => 0,      # maximum width
		'max_h' => 0,      # maximum height
		'type'  => 'file', # file : image only / thumb : thumbnail only / link : with link to image
		'xml'  => 1,       # output as xml format or not
		'attr'  => undef,  # extra attribute
		@_,
	);
	my %attribute = (
		'width'  => ICON_WIDTH,
		'height' => ICON_HEIGHT,
		'src'    => '',
		'title'  => $self->name(),
		'alt'    => $self->name(),
		'class'  => IMAGE_CLASS,
	);
	if ( $param{'attr'} and ref($param{'attr'}) eq 'HASH' )
	{
		map { $attribute{$_} = $param{'attr'}->{$_} } keys %{$param{'attr'}}
	}
	if ( !$self->is_image() )
	{ # it is not image, so type icon will be shown.
		my $dir = sb::Config->get()->value('dir_temp') . PARTS_DIR;
		my $srv = sb::Config->get()->value('srv_temp') . PARTS_DIR;
		my $tail = $self->object('file')->extension();
		$tail =~ s/^\.//g;
		my $icon = $tail . ICON_SUFFIX;
		$icon = ICON_UNDEFINED if (!-e $dir . $icon);
		$attribute{'src'} = $srv . $icon;
	}
	else
	{
		if ($param{'type'} ne 'file' and $self->thumb ne '')
		{ # if it has a thumbnail image
			my ($w,$h) = $self->get_size(%param,'type'=>'thumb');
			$attribute{'width'}  = $w;
			$attribute{'height'} = $h;
			$attribute{'class'}  = THUMB_CLASS;
			$attribute{'src'}    = $self->get_url('thumb');
		}
		else
		{ # no thumbnail image
			my ($w,$h) = $self->get_size(%param,'type'=>'file');
			$attribute{'width'}  = $w;
			$attribute{'height'} = $h;
			$attribute{'src'}    = $self->get_url();
		}
	}
	my $tag = '<img ';
	map {
		$tag .= $_ . '="' . $attribute{$_} . '" ' if ( defined $attribute{$_} );
	} keys %attribute;
	$tag .= ($param{'xml'}) ? '/>' : '>';
	return ($param{'type'} eq 'link') ? '<a href="' . $self->get_url() . '">' . $tag  . '</a>' : $tag;
}
sub get_as_mime
{
	my $self = shift;
	my $type = shift;
	return if ($type eq 'thumb' and $self->thumb eq '');
	return $self->object($type)->get_as_mime();
}
sub is_image
{
	my $self = shift;
	return $self->object('file')->is_image();
}
sub get_content_type
{
	my $self = shift;
	return $self->object('file')->content_type();
}
sub get_size
{
	my $self = shift;
	my %param = (
		'max_w' => 0,      # maximum width
		'max_h' => 0,      # maximum height
		'type'  => 'file', # file or thumb
		@_,
	);
	my $obj = $self->object($param{'type'});
	if ( !$obj
	  or !$obj->is_image()
	  or ($param{'type'} eq 'thumb' and $self->thumb() eq ''))
	{
		return('--','--');
	}
	return $obj->get_size(%param);
}
sub get_filesize
{
	my $self = shift;
	my $size = $self->object('file')->filesize() / 1024;
	return sprintf("%.2f",$size);
}
sub get_path
{
	my $self = shift;
	my $type = $self->_check_type(shift);
	return sb::Config->get()->value('conf_dir_base') . $self->dir() . $self->$type();
}
sub get_url
{
	my $self = shift;
	my $type = $self->_check_type(shift);
	return sb::Config->get()->value('conf_srv_base') . $self->dir() . $self->$type();
}
# --------------------------------------
# // private functions
sub _check_type
{
	my $self = shift;
	my $type = shift;
	$type = 'file' if ($type eq 'thumb' and $self->thumb() eq '');
	$type = 'file' if ($type ne 'file' and $type ne 'thumb');
	return $type;
}
1;
__END__
